#include <stdio.h>
#include <sys/timeb.h>
#include <time.h>
#include <errno.h>
#include <va/va.h>
#include <string.h>
#include <assert.h>

#include "acquisition.h"

//acquisition code below was copied from xiViewer
static SETUP	setup;
LPSETUP lps = &setup;
HANDLE	hMV = INVALID_HANDLE_VALUE;

void	setT0(void){
	lps->pcn0 = timeGetTime();
}
double	getT(void){
	lps->pcn = timeGetTime();
	return (double)(lps->pcn-lps->pcn0) / 1000;
}
#define LUTEN 0
void* acquisition(void* args)
{
HANDLE	hMVlocal = INVALID_HANDLE_VALUE;

LPWORD	pRawData;
DWORD	dwSerNum;
MM40_RETURN	mvret;
int iPrevFrame = -1;
LUT lut;
float tpfi0, tpfie, tpfr0, tpfre;

	XExposeEvent xev;
	Window win = (Window)args;
	xev.type = Expose;
	xev.display = x11_display;
	xev.window = win;
	xev.count = 0;

	lps->bRun = TRUE; 

	while (lps->bRun) {
		if (hMVlocal == INVALID_HANDLE_VALUE) {
			mm40GetDevice( 0, &dwSerNum );
			mm40Initialize("", dwSerNum, &hMVlocal );
			if (  mmInitializeHardware( hMVlocal ) != MM40_OK ) { 
				lps->bRun = FALSE;
				break;
			}
			mmSetExtention(hMVlocal, EXT_RAW_EN, 1);
			//mmApplyCameraProfile( hMVlocal ); XXX: not implemeted in Linux version of library yet
			mmSetGainVal(hMVlocal, lps->fGain);

			lut.wVersion = LUT_STRUCT_VER;
			lut.wLenght = LUT_STRUCT_LEN;
			lut.bUseDefault = TRUE;
			mmSetLUT(hMVlocal, &lut);
#if LUTEN
			mmSetExtention(hMVlocal, EXT_LUT, 1);	
#endif
			if ((mvret = mmSetMode( hMVlocal, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy)) != MM40_OK){
				lps->bRun = FALSE;
				continue;
			}

			while(!mmIsSetModeComplete(hMVlocal));

			lps->fFPS = 100;
			lps->fFPS_sum = 100;
			mmGetModeTiming(hMVlocal, &tpfi0, &tpfie, &tpfr0, &tpfre);
			if (tpfie && tpfre) {
				lps->fFPS = (float)(1./tpfre);
				lps->fFPS_sum = lps->fFPS;
			}
			
			if (mmStartAcquisition( hMVlocal ) != MM40_OK ) {
				lps->bRun = FALSE;
				break;
			}
		}
		if ( hMVlocal != INVALID_HANDLE_VALUE ) {
			while ( 1 ) {
				if (!lps->bRun) break;
				mvret = mmWaitForField( hMVlocal, 1000 );	//$$$
				if ( mvret == MM40_OK ) break;
				Sleep(0);
				if ( mvret == MM40_INVALID_HANDLE ) {
					mmStopAcquisition( hMVlocal );
					hMVlocal = INVALID_HANDLE_VALUE;
					break;
				}
				if ( mvret == MM40_NO_IMAGE ) continue;
			}
		}

		if (!lps->bRun) break;

		if ( hMVlocal == INVALID_HANDLE_VALUE )
			continue;

		pRawData=NULL;

		while( mmIsFieldReady(hMVlocal) == MM40_OK ) { // This 'while' is to grab all the fields standing in the queue
			if ( hMVlocal != INVALID_HANDLE_VALUE ) {

				if (mmAcquireField( hMVlocal, (HANDLE*)&pRawData ) != MM40_OK) {
					continue;
				}
			}
			if (pRawData==NULL) continue;
			mmUpdateBayerFrame( hMVlocal, pRawData );
			mmRetrieveFrameBayerEx(hMVlocal, &lps->gf);
			lps->dwNframes++;
			if(lps->gf.curFN - (iPrevFrame + 1) < 1000) lps->dwLframes += (lps->gf.curFN - (iPrevFrame + 1));
			iPrevFrame = lps->gf.curFN;
		}

		if (!mmIsFrameComplete(hMVlocal)) {
			continue;
		}

		lps->bHaveData = TRUE;
		if (lps->bRun) {
			XSendEvent(x11_display, win, True, ExposureMask, (XEvent*)&xev);
			XFlush(x11_display);
		}
	}

	if (hMVlocal != INVALID_HANDLE_VALUE) {
		mmStopAcquisition( hMVlocal );
		mm40Uninitialize( hMVlocal );
	}
	XDestroyWindow(x11_display, win);
	XFlush(x11_display);
	return 0;
}

void setupCam() {
	U32		devices;
	DWORD	dwSerial;

	memset(lps, 0, sizeof(SETUP));
	lps->bDrawImage = TRUE;
		lps->uiExposure = 10000;
		lps->uiMode = 0;
		lps->fGain = 0;
		lps->bDrawImage = TRUE;
		lps->zoom = 2;
	lps->show = TRUE;

	mm40GetNumberDevices("", &devices );
	if (devices) {
		mm40GetDevice( 0, &dwSerial );
		mm40Initialize("", dwSerial, &hMV );
		if (hMV == INVALID_HANDLE_VALUE) {
		} else {
			mmInitializeHardware( hMV );
			//mmApplyCameraProfile( hMV );
			lps->dwNframes = 0;
			lps->dwPframes = 0;
			lps->dwLframes = 0;
			lps->dwTime = 0;
		}
	}
}

/*void DrawImage()
{

	if(lps->sgf.bp != lps->gf.bp) lps->bProc16to8 = FALSE;
	
	memcpy(&lps->sgf, &lps->gf, sizeof(MMGETFRAMEEX));

	if(lps->sgf.modeRDO & RP_PS_MONO8){

	}else{
		char *data = (char*)lps->sgf.bp;
		char * byte_data = (char*)lps->sgf.bp;
		int bitshift = lps->sgf.curBPP - 8;
		int cnt = (lps->sgf.curCX * lps->sgf.curCY * sizeof(WORD)) / 32;
		
		if(!lps->bProc16to8){
			_asm{
				mov			esi,		[data]				;
				mov			edi,		[byte_data]			;
				movd		xmm2,		[bitshift]			;
			t00:	//	
				movdqa		xmm0,		qword ptr[esi]		;
				movdqa		xmm1,		qword ptr[esi+16]	;
				psrlw		xmm0,		xmm2				;
				psrlw		xmm1,		xmm2				;
				packuswb	xmm0,		xmm1				;
				movntdq		[edi],		xmm0				;
				add			edi,		16					;
				add			esi,		32					;

				sub			[cnt],		1					;
				ja			t00								;	
			}
			lps->bProc16to8 = TRUE;
		}
	}

	else SetDIBitsToDevice(hDC, lps->x0, lps->y0, lps->cx, lps->cy, 0, 0, 0, lps->sgf.curCY, lps->sgf.bp, (LPBITMAPINFO)&lps->bi, DIB_PAL_COLORS);

}*/

int drawImage(VADisplay va_dpy, VASurfaceID surface_id, /*VAImageFormat *image_format,*/ int surface_width, int surface_height,
		Window win, int win_width, int win_height) {
	VAImage surface_image;
	char *surface_p;
	VAStatus va_status;
	int i;

	if(!lps->show)
		return 1;
	/*va_status = vaCreateImage(va_dpy, image_format, surface_width, surface_height, &surface_image);
	CHECK_VASTATUS(va_status, "vaCreateImage");*/
	va_status = vaDeriveImage(va_dpy, surface_id, &surface_image);
	CHECK_VASTATUS(va_status, "vaDeriveImage");
	if(surface_image.width < surface_width || surface_image.height < surface_height) {
		printf("Didn't get expected image size: %dx%d!\n", surface_image.width, surface_image.height);
		return 0;
	}
	if(surface_image.format.fourcc != VA_FOURCC_NV12) {
		printf("Didn't get expected image format: %x\n", surface_image.format.fourcc);
		return 0;
	}
	va_status = vaMapBuffer(va_dpy, surface_image.buf, (void**)&surface_p);
	CHECK_VASTATUS(va_status, "vaMapBuffer");
	if(!lps->bProc16to8 && !(lps->sgf.modeRDO & RP_PS_MONO8)) {
		char *data = (char*)lps->sgf.bp;
		char * byte_data = (char*)lps->sgf.bp;
		int bitshift = lps->sgf.curBPP - 8;
		int cnt = (lps->sgf.curCX * lps->sgf.curCY * sizeof(WORD)) / 32;
		_asm{
			mov			esi,		[data]				;
			mov			edi,		[byte_data]			;
			movd		xmm2,		[bitshift]			;
t00:
			movdqa		xmm0,		[esi]		;
			movdqa		xmm1,		[esi+16]	;
			psrlw		xmm0,		xmm2				;
			psrlw		xmm1,		xmm2				;
			packuswb	xmm0,		xmm1				;
			movntdq		[edi],		xmm0				;
			add			edi,		16					;
			add			esi,		32					;

			sub			[cnt],		1					;
			ja			t00								;
		}
		lps->bProc16to8 = TRUE;
	}
	for(i=0;i<surface_height;i++) {
		memcpy(surface_p+surface_image.offsets[0]+i*surface_image.pitches[0], (char*)lps->sgf.bp+i*surface_width, surface_width);
	}
	memset(surface_p + surface_image.offsets[1], 128, surface_image.pitches[1]*surface_image.height/2);
	/*va_status = vaPutImage(va_dpy, surface_id, surface_image.image_id, 0, 0, surface_width, surface_height, 0, 0, surface_width, surface_height);
	CHECK_VASTATUS(va_status, "vaPutImage");*/
	va_status = vaUnmapBuffer(va_dpy,surface_image.buf);
	CHECK_VASTATUS(va_status, "vaUnmapBuffer");
	va_status = vaDestroyImage(va_dpy,surface_image.image_id);
	CHECK_VASTATUS(va_status, "vaDestroyImage");
	va_status = vaPutSurface(va_dpy, surface_id, win, 0, 0, surface_width, surface_height, 0, 0, win_width, win_height, 0, 0, VA_FRAME_PICTURE);
	CHECK_VASTATUS(va_status,"vaPutSurface");
	va_status = vaSyncSurface(va_dpy, surface_id);
	CHECK_VASTATUS(va_status, "vaSyncSurface");
	return 1;
}
