#include <stdio.h>
#include <sys/time.h>
#include <pthread.h>

#include <gtk/gtk.h>

#include "acquisition.h"

//maximum size of video window in percentage of screen size
#define MAX_SIZE 0.8
//window title
#define WIN_NAME "vaViewer"

#define REMOVE_EVENTS(display, w, mask, event) while(XCheckWindowEvent(display, w, mask, event)){}

Display *x11_display;

void stopLive() {
	if(!lps->bRun) return;
	lps->bRun = FALSE;
	lps->bHaveData = FALSE;
}

void resize_window(Window win, int width, int height) {
	XSizeHints sizehints;
	sizehints.min_aspect.x = width;
	sizehints.min_aspect.y = height;
	sizehints.max_aspect.x = width;
	sizehints.max_aspect.y = height;
	sizehints.flags = PAspect;
	XSetWMNormalHints(x11_display, win, &sizehints);
	XResizeWindow(x11_display, win, width, height);
}

Window create_window(int width, int height) {
	Window root, win;
	XTextProperty winname;
	char win_name[] = WIN_NAME;
	char* pwin_name = &win_name[0];
	Atom wmDeleteMessage = XInternAtom(x11_display, "WM_DELETE_WINDOW", False);

	XmbTextListToTextProperty(x11_display, &pwin_name, 1, XStringStyle, &winname);
	root = RootWindow(x11_display, DefaultScreen(x11_display));
	win = XCreateSimpleWindow(x11_display, root, 0, 0, width, height, 0, 0, WhitePixel(x11_display, DefaultScreen(x11_display)));
	if(win) {
		resize_window(win, width, height);
		XSetWMName(x11_display, win, &winname);
		XSetIconName(x11_display, win, "vaViewer");
		XSelectInput(x11_display, win, ExposureMask|/*KeyPressMask|*/StructureNotifyMask);
		XSetWMProtocols(x11_display, win, &wmDeleteMessage, 1);
		XMapWindow(x11_display, win);
		XSync(x11_display, False);
	}
	return win;
}

void* videoDisplay(void* args)
{
    XEvent event;
	Window win;
	int redraw;
	VAStatus va_status;
	pthread_t acqThread;
	VASurfaceID surface_id;
	unsigned int start_time;
	//VAImageFormat *va_image_formats;
	//int i, max_image_formats, va_num_image_formats;
	int win_width=640, win_height=480;
	//VAImageFormat image_format;
	VADisplay va_dpy;
	unsigned int surface_width=0, surface_height=0;
	unsigned int dpy_width=800, dpy_height=600;
	int major_ver, minor_ver;

	dpy_width = DisplayWidth(x11_display, DefaultScreen(x11_display));
	dpy_height = DisplayHeight(x11_display, DefaultScreen(x11_display));
	va_dpy = vaGetDisplay(x11_display);
	va_status = vaInitialize(va_dpy, &major_ver, &minor_ver);
	CHECK_VASTATUS(va_status, "vaInitialize");
	/*//find NV12 image format
	max_image_formats = vaMaxNumImageFormats(va_dpy);
	va_image_formats = (VAImageFormat*)calloc(max_image_formats, sizeof(*va_image_formats));
	if(!va_image_formats)
		return 0;
	va_status = vaQueryImageFormats(va_dpy, va_image_formats, &va_num_image_formats);
	CHECK_VASTATUS(va_status, "vaQueryImageFormats");
	for(i=0;i<va_num_image_formats;i++) {
		if(va_image_formats[i].fourcc == VA_FOURCC_NV12 || va_image_formats[i].fourcc == VA_FOURCC_YV12) {
			memcpy(&image_format, &va_image_formats[i], sizeof(image_format));
			break;
		}
	}
	free(va_image_formats);
	if(i == va_num_image_formats) {
		fprintf(stderr, "VAAPI implementation doesn't support NV12 image format!\n");
		return 0;
	}*/

	XSync(x11_display, TRUE); //clear event queue
	win = create_window(win_width, win_height);
	if(!win)
		return 0;

	if(pthread_create(&acqThread, NULL, acquisition, (void*)win))
		return 0;

	while(1) {
		redraw = 0;
		start_time = timeGetTime();
		XNextEvent(x11_display, &event);
		/* bail on any focused key press */
		if(/*event.type == KeyPress || */event.type == DestroyNotify ||
				(event.type == ClientMessage && (Atom)event.xclient.data.l[0] == XInternAtom(x11_display, "WM_DELETE_WINDOW", False)))
			break;
		/* rescale the video to fit the window */
		if(event.type == ConfigureNotify) {
			win_width = event.xconfigure.width;
			win_height = event.xconfigure.height;
			redraw = 1;
		}
		/* update picture */
		if(event.type == Expose && event.xexpose.count == 0) {
			REMOVE_EVENTS(x11_display, win, ExposureMask, &event)
			redraw = 1;
		}
		if(redraw && lps->bHaveData) {
			if(lps->sgf.bp != lps->gf.bp) lps->bProc16to8 = FALSE;
			memcpy(&lps->sgf, &lps->gf, sizeof(MMGETFRAMEEX));
			if(surface_width != lps->sgf.curCX || surface_height != lps->sgf.curCY) {
				if(surface_width) {
					vaDestroySurfaces(va_dpy, &surface_id, 1);
					CHECK_VASTATUS(va_status, "vaDestroySurfaces");
				}
				surface_width = lps->sgf.curCX;
				surface_height = lps->sgf.curCY;
				printf("Image size changed to %dx%d\n", surface_width, surface_height);
				va_status = vaCreateSurfaces(va_dpy, surface_width, surface_height, VA_RT_FORMAT_YUV420, 1, &surface_id);
				CHECK_VASTATUS(va_status, "vaCreateSurfaces");
				win_width = surface_width;
				win_height = surface_height;
				while(win_width > dpy_width*MAX_SIZE || win_height > dpy_height*MAX_SIZE) {
					win_width /= 2;
					win_height /= 2;
				}
				resize_window(win, win_width, win_height);
			}
			if(!drawImage(va_dpy, surface_id, /*&image_format,*/ surface_width, surface_height, win, win_width, win_height))
				break;
			if(lps->putsurface_time > 1000) {
				double	tau = 0.5;
				char win_name[256];
				XTextProperty winname;
				char* pwin_name = &win_name[0];

				lps->fTPF = getT();
				setT0();
				if (lps->dwNframes != lps->dwPframes && lps->dwNframes) {
					lps->fTPF /= (double)(lps->dwNframes - lps->dwPframes);
				}
				lps->dwPframes = lps->dwNframes;
				if(lps->fTPF) lps->fFPS = 1./lps->fTPF;
				if (lps->dwNframes) lps->fFPS_sum = lps->fFPS_sum*tau + lps->fFPS*(1.-tau);
				else		      lps->fFPS_sum = lps->fFPS;
				lps->dwTime++;
				snprintf(&win_name[0], 256, "Video FPS: %.1f; Acquisition: frm %lu skip %lu fps %.1f", lps->frame_num*1000.0/(float)lps->putsurface_time, lps->dwNframes, lps->dwLframes, (float)lps->fFPS_sum);
				lps->putsurface_time = 0;
				lps->frame_num = 0;
				XmbTextListToTextProperty(x11_display, &pwin_name, 1, XStringStyle, &winname);
				XSetWMName(x11_display, win, &winname);
			}
			lps->putsurface_time += (timeGetTime() - start_time);
			lps->frame_num++;
		}
	}
	stopLive();
	pthread_join(acqThread, NULL);
	vaDestroySurfaces(va_dpy, &surface_id, 1);
	CHECK_VASTATUS(va_status, "vaDestroySurfaces");
	vaTerminate(va_dpy);
	CHECK_VASTATUS(va_status, "vaTerminate");
	//gtk_main_quit();
	lps->videoThread = 0;
	return (void*)1;
}

void startLive() {
	if(lps->videoThread)
		return;
	lps->bHaveData = FALSE;
	setT0();
	if(pthread_create(&lps->videoThread, NULL, videoDisplay, NULL))
		exit(1);
}

typedef struct {
    GtkWidget *window, *boxmain, *boxgpi, *boxx, *boxy, *gpi1, *gpi2, *gpi3, *gpi4, *mode,
			  *labelexp, *exp, *labelgain, *gain, *labelx0, *x0, *labely0, *y0, *labelcx, *cx, *labelcy, *cy, *show, *run;
} ctrl_window;

gboolean time_handler(ctrl_window *ctrl) {
	DWORD level = 0;

	if(!lps->bRun){
		if (hMV != INVALID_HANDLE_VALUE){
			mmGetGPIO(hMV, 1, NULL, &level);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi1), level);
			mmGetGPIO(hMV, 2, NULL, &level);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi2), level);
			mmGetGPIO(hMV, 3, NULL, &level);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi3), level);
			mmGetGPIO(hMV, 4, NULL, &level);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi4), level);
		}
	}else{
		if (hMV != INVALID_HANDLE_VALUE){
			mmGetGPIO(hMV, 0xFFFFFFFF, NULL, &level);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi1), level & 1);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi2), (level >> 1) & 1);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi3), (level >> 2) & 1);
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->gpi4), (level >> 3) & 1);
		}
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl->run), lps->bRun);
	return TRUE;
}

gboolean update_x0(GtkAdjustment *adj, gpointer) {
	lps->m_roi_x0 = gtk_adjustment_get_value(adj);
	if(lps->m_roi_cx + lps->m_roi_x0 > lps->dwCx){
		lps->m_roi_x0 = lps->dwCx - lps->m_roi_cx;
		gtk_adjustment_set_value(adj, lps->m_roi_x0);
	}
	if(lps->m_roi_x0 == 0 && lps->m_roi_y0 == 0 && lps->m_roi_cx == lps->dwCx && lps->m_roi_cy == lps->dwCy) {
		lps->dwROIx0 = 0, lps->dwROIy0 = 0, lps->dwROIcx = 0, lps->dwROIcy = 0;
	} else {
		lps->dwROIx0 = lps->m_roi_x0;
		lps->dwROIy0 = lps->m_roi_y0;
		lps->dwROIcx = lps->m_roi_cx;
		lps->dwROIcy = lps->m_roi_cy;
	}
	mmSetMode(hMV, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy);
	return TRUE;
}

gboolean update_y0(GtkAdjustment *adj, gpointer) {
	lps->m_roi_y0 = gtk_adjustment_get_value(adj);
	if(lps->m_roi_cy + lps->m_roi_y0 > lps->dwCy){
		lps->m_roi_y0 = lps->dwCy - lps->m_roi_cy;
		gtk_adjustment_set_value(adj, lps->m_roi_y0);
	}
	if(lps->m_roi_x0 == 0 && lps->m_roi_y0 == 0 && lps->m_roi_cx == lps->dwCx && lps->m_roi_cy == lps->dwCy) {
		lps->dwROIx0 = 0, lps->dwROIy0 = 0, lps->dwROIcx = 0, lps->dwROIcy = 0;
	} else {
		lps->dwROIx0 = lps->m_roi_x0;
		lps->dwROIy0 = lps->m_roi_y0;
		lps->dwROIcx = lps->m_roi_cx;
		lps->dwROIcy = lps->m_roi_cy;
	}
	mmSetMode(hMV, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy);
	return TRUE;
}

gboolean update_cx(GtkAdjustment *adj, gpointer) {
	lps->m_roi_cx = gtk_adjustment_get_value(adj);
	if(lps->m_roi_x0 + lps->m_roi_cx > lps->dwCx){
		lps->m_roi_cx = lps->dwCx - lps->m_roi_x0;
		gtk_adjustment_set_value(adj, lps->m_roi_cx);
	}
	if(lps->m_roi_x0 == 0 && lps->m_roi_y0 == 0 && lps->m_roi_cx == lps->dwCx && lps->m_roi_cy == lps->dwCy) {
		lps->dwROIx0 = 0, lps->dwROIy0 = 0, lps->dwROIcx = 0, lps->dwROIcy = 0;
	} else {
		lps->dwROIx0 = lps->m_roi_x0;
		lps->dwROIy0 = lps->m_roi_y0;
		lps->dwROIcx = lps->m_roi_cx;
		lps->dwROIcy = lps->m_roi_cy;
	}
	mmSetMode(hMV, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy);
	return TRUE;
}

gboolean update_cy(GtkAdjustment *adj, gpointer) {
	lps->m_roi_cy = gtk_adjustment_get_value(adj);
	if(lps->m_roi_y0 + lps->m_roi_cy > lps->dwCy){
		lps->m_roi_cy = lps->dwCy - lps->m_roi_y0;
		gtk_adjustment_set_value(adj, lps->m_roi_cy);
	}
	if(lps->m_roi_x0 == 0 && lps->m_roi_y0 == 0 && lps->m_roi_cx == lps->dwCx && lps->m_roi_cy == lps->dwCy) {
		lps->dwROIx0 = 0, lps->dwROIy0 = 0, lps->dwROIcx = 0, lps->dwROIcy = 0;
	} else {
		lps->dwROIx0 = lps->m_roi_x0;
		lps->dwROIy0 = lps->m_roi_y0;
		lps->dwROIcx = lps->m_roi_cx;
		lps->dwROIcy = lps->m_roi_cy;
	}
	mmSetMode(hMV, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy);
	return TRUE;
}

gboolean update_exposure(GtkAdjustment *adj, gpointer) {
	unsigned int exposure = 1000*gtk_adjustment_get_value(adj);
	if(exposure != lps->uiExposure) {
		lps->uiExposure = exposure;
		mmSetMode(hMV, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy);
	}
	return TRUE;
}

gboolean update_gain(GtkAdjustment *adj, gpointer) {
	float gain = gtk_adjustment_get_value(adj);
	if(gain != lps->fGain) {
		lps->fGain = gain;
		mmSetGainVal(hMV, lps->fGain);
		mmSetMode(hMV, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy);
	}
	return TRUE;
}

gboolean update_mode(GtkComboBox *mode, ctrl_window *ctrl) {
	float fL, fH;
	DWORD	dwStops, dwDefGain;
	MMMODE mmmode;

	lps->uiMode = gtk_combo_box_get_active(mode);
	if(mmGetModeDescriptor(hMV, lps->uiMode, &mmmode) != MM40_OK) {
		printf("Couldn't get mode description!\n");
		exit(-1);
	}
	lps->m_roi_cx = lps->dwROIcx = lps->dwCx = mmmode.dwCx;
	lps->m_roi_cy = lps->dwROIcy = lps->dwCy = mmmode.dwCy;
	lps->m_roi_x0 = lps->dwROIx0 = 0;
	lps->m_roi_y0 = lps->dwROIy0 = 0;
	mmSetMode(hMV, lps->uiExposure, lps->uiMode, lps->dwROIx0, lps->dwROIy0, lps->dwROIcx, lps->dwROIcy);
	mmGetGainInfo(hMV , &fL, &fH, &dwStops, &dwDefGain);
	if(fL > lps->fGain) lps->fGain = fL;
	gtk_adjustment_configure(gtk_range_get_adjustment(GTK_RANGE(ctrl->gain)), lps->fGain, fL, fH, 0.1, 1, 0);
	gtk_adjustment_configure(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl->x0)), 0, 0, lps->dwCx-4, 2, 20, 0);
	gtk_adjustment_configure(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl->y0)), 0, 0, lps->dwCy-2, 2, 20, 0);
	gtk_adjustment_configure(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl->cx)), lps->dwCx, 4, lps->dwCx, 4, 20, 0);
	gtk_adjustment_configure(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl->cy)), lps->dwCy, 2, lps->dwCy, 2, 20, 0);
	return TRUE;
}

gboolean update_show(GtkToggleButton *show, gpointer) {
	lps->show = gtk_toggle_button_get_active(show);
	return TRUE;
}

gboolean update_run(GtkToggleButton *run, gpointer) {
	if(gtk_toggle_button_get_active(run)) {
		startLive();
	} else {
		stopLive();
	}
	return TRUE;
}

int main(int argc,char **argv)
{
	ctrl_window ctrl;
	char szMode[MAX_PATH];

	setvbuf(stdout, NULL, _IONBF, 0); //disable bufferring on stdout
	XInitThreads(); //make Xlib thread-safe
	//init X11
	x11_display = XOpenDisplay(NULL);
	if(x11_display == NULL) {
		printf("Can't connect to X server!\n");
		exit(-1);
	}
	//start
	setupCam();
	if(hMV == INVALID_HANDLE_VALUE) {
		printf("Couldn't setup camera!\n");
		exit(-1);
	}
	startLive();

	gtk_init(&argc, &argv);
	//create the main, top level, window
	ctrl.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	//give it the title
	gtk_window_set_title (GTK_WINDOW(ctrl.window), "vaViewer control");
	/* Connect the destroy signal of the window to gtk_main_quit
	 * When the window is about to be destroyed we get a notification and
	 * stop the main GTK+ loop
	 */
	g_signal_connect(ctrl.window, "destroy", G_CALLBACK(gtk_main_quit), NULL);
	//create widgets
	ctrl.boxmain = gtk_vbox_new(FALSE, 0);
	ctrl.boxgpi = gtk_hbox_new(TRUE, 0);
	ctrl.boxx = gtk_hbox_new(FALSE, 0);
	ctrl.boxy = gtk_hbox_new(FALSE, 0);
	ctrl.labelexp = gtk_label_new("Exposure (ms)");
	ctrl.labelgain = gtk_label_new("Gain (dB)");
	ctrl.labelx0 = gtk_label_new("x0");
	ctrl.labelcx = gtk_label_new("cx");
	ctrl.labely0 = gtk_label_new("y0");
	ctrl.labelcy = gtk_label_new("cy");
	ctrl.gpi1 = gtk_check_button_new_with_label("GPI1");
	ctrl.gpi2 = gtk_check_button_new_with_label("GPI2");
	ctrl.gpi3 = gtk_check_button_new_with_label("GPI3");
	ctrl.gpi4 = gtk_check_button_new_with_label("GPI4");
	ctrl.mode = gtk_combo_box_new_text();
	ctrl.exp = gtk_hscale_new_with_range(1, 1000, 1);
	ctrl.gain = gtk_hscale_new_with_range(0, 1, 0.1); //use dummy limits
	ctrl.x0 = gtk_spin_button_new_with_range(0, 128, 2); //use dummy max limit
	ctrl.y0 = gtk_spin_button_new_with_range(0, 128, 2); //use dummy max limit
	ctrl.cx = gtk_spin_button_new_with_range(4, 128, 4); //use dummy max limit
	ctrl.cy = gtk_spin_button_new_with_range(2, 128, 2); //use dummy max limit
	ctrl.show = gtk_toggle_button_new_with_label("Live view");
	ctrl.run = gtk_toggle_button_new_with_label("Acquisition");
	//tune them
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl.show), TRUE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ctrl.run), TRUE);
	gtk_widget_set_sensitive(ctrl.gpi1, FALSE);
	gtk_widget_set_sensitive(ctrl.gpi2, FALSE);
	gtk_widget_set_sensitive(ctrl.gpi3, FALSE);
	gtk_widget_set_sensitive(ctrl.gpi4, FALSE);
	gtk_scale_set_digits(GTK_SCALE(ctrl.exp), 0);
	gtk_range_set_update_policy(GTK_RANGE(ctrl.exp), GTK_UPDATE_DISCONTINUOUS); //deprecated
	gtk_range_set_update_policy(GTK_RANGE(ctrl.gain), GTK_UPDATE_DISCONTINUOUS); //deprecated
	gtk_adjustment_set_value(gtk_range_get_adjustment(GTK_RANGE(ctrl.exp)), 10);
	gtk_scale_set_value_pos(GTK_SCALE(ctrl.exp), GTK_POS_RIGHT);
	gtk_scale_set_value_pos(GTK_SCALE(ctrl.gain), GTK_POS_RIGHT);
	for(int i=0;;i++) {
		if(mmGetModeDescriptorString(hMV, i, szMode, MAX_PATH) != MM40_OK){ //gets mode description
			break;
		}
		gtk_combo_box_append_text(GTK_COMBO_BOX(ctrl.mode), szMode);
	}
	gtk_combo_box_set_active(GTK_COMBO_BOX(ctrl.mode), 0);
	//pack everything into window
	gtk_container_add(GTK_CONTAINER(ctrl.boxgpi), ctrl.gpi1);
	gtk_container_add(GTK_CONTAINER(ctrl.boxgpi), ctrl.gpi2);
	gtk_container_add(GTK_CONTAINER(ctrl.boxgpi), ctrl.gpi3);
	gtk_container_add(GTK_CONTAINER(ctrl.boxgpi), ctrl.gpi4);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.boxgpi);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.mode);
	gtk_container_add(GTK_CONTAINER(ctrl.boxx), ctrl.labelx0);
	gtk_container_add(GTK_CONTAINER(ctrl.boxx), ctrl.x0);
	gtk_container_add(GTK_CONTAINER(ctrl.boxy), ctrl.labely0);
	gtk_container_add(GTK_CONTAINER(ctrl.boxy), ctrl.y0);
	gtk_container_add(GTK_CONTAINER(ctrl.boxx), ctrl.labelcx);
	gtk_container_add(GTK_CONTAINER(ctrl.boxx), ctrl.cx);
	gtk_container_add(GTK_CONTAINER(ctrl.boxy), ctrl.labelcy);
	gtk_container_add(GTK_CONTAINER(ctrl.boxy), ctrl.cy);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.boxx);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.boxy);	
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.labelexp);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.exp);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.labelgain);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.gain);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.show);
	gtk_container_add(GTK_CONTAINER(ctrl.boxmain), ctrl.run);
	gtk_container_add(GTK_CONTAINER(ctrl.window), ctrl.boxmain);
	update_mode(GTK_COMBO_BOX(ctrl.mode), &ctrl);
	//show window
	gtk_widget_show_all(ctrl.window);
	//register handlers
	g_timeout_add(1000, (GSourceFunc)time_handler, (gpointer)&ctrl);
	g_signal_connect(gtk_range_get_adjustment(GTK_RANGE(ctrl.gain)), "value_changed", G_CALLBACK(update_gain), NULL);
	g_signal_connect(gtk_range_get_adjustment(GTK_RANGE(ctrl.exp)), "value_changed", G_CALLBACK(update_exposure), NULL);
	g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl.x0)), "value_changed", G_CALLBACK(update_x0), NULL);
	g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl.y0)), "value_changed", G_CALLBACK(update_y0), NULL);
	g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl.cx)), "value_changed", G_CALLBACK(update_cx), NULL);
	g_signal_connect(gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(ctrl.cy)), "value_changed", G_CALLBACK(update_cy), NULL);
	g_signal_connect(ctrl.show, "toggled", G_CALLBACK(update_show), NULL);
	g_signal_connect(ctrl.run, "toggled", G_CALLBACK(update_run), NULL);
	g_signal_connect(ctrl.mode, "changed", G_CALLBACK(update_mode), (gpointer)&ctrl);
	//start the main loop
	gtk_main();
	//exit
	lps->bRun = FALSE;
	lps->bHaveData = FALSE;
	pthread_join(lps->videoThread, NULL);
	mm40Uninitialize(hMV);
	XCloseDisplay(x11_display);
	return 0;
}
